# pywws - Python software for USB Wireless Weather Stations
# http://github.com/jim-easterbrook/pywws
# Copyright (C) 2008-18  pywws contributors

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Store parameters in easy to access files, and access backend data

Introduction
------------

This module is at the core of pywws. By default it stores data on disc,
using a backend module which uses text files (see filedata.py) but
other plugin backend modules can be used to use alternative means.
These modules must adopt the same API (Class names and methods) as
filedata.py so as to be transparent to the rest of pywws.

From a "user" point of view, the data is accessed as a cross between a
list and a dictionary. Each data record is indexed by a
:py:class:`datetime.datetime` object (dictionary behaviour), but
records are stored in order and can be accessed as slices (list
behaviour).

For example, to access the hourly data for Christmas day 2009, one
might do the following::

  from datetime import datetime
  import pywws.storage
  datastore = pywws.storage.PywwsContext('weather_data', False)
  hourly = datastore.hourly_data
  for data in hourly[datetime(2009, 12, 25):datetime(2009, 12, 26)]:
      print(data['idx'], data['temp_out'])

Some more examples of data access::

  # get value nearest 9:30 on Christmas day 2008
  data[data.nearest(datetime(2008, 12, 25, 9, 30))]
  # get entire array, equivalent to data[:]
  data[datetime.min:datetime.max]
  # get last 12 hours worth of data
  data[datetime.utcnow() - timedelta(hours=12):]

Note that the :py:class:`datetime.datetime` index is in UTC. You may
need to apply an offset to convert to local time.

See filedata.py for more details on the underlying data store API

Detailed API
------------

"""

from __future__ import with_statement

from ast import literal_eval
from contextlib import contextmanager
import logging
import os
import sys
import threading
import importlib

if sys.version_info[0] >= 3:
    from configparser import RawConfigParser
else:
    from ConfigParser import RawConfigParser

from pywws.weatherstation import WSDateTime
logger = logging.getLogger(__name__)


class ParamStore(object):
    def __init__(self, root_dir, file_name):
        self._lock = threading.Lock()
        with self._lock:
            if not os.path.isdir(root_dir):
                raise RuntimeError(
                    'Directory "' + root_dir + '" does not exist.')
            self._path = os.path.join(root_dir, file_name)
            self._dirty = False
            # open config file
            self._config = RawConfigParser()
            self._config.read(self._path)

    def flush(self):
        if not self._dirty:
            return
        with self._lock:
            self._dirty = False
            with open(self._path, 'w') as of:
                self._config.write(of)

    def get(self, section, option, default=None):
        """Get a parameter value and return a string.

        If default is specified and section or option are not defined
        in the file, they are created and set to default, which is
        then the return value.

        """
        with self._lock:
            if not self._config.has_option(section, option):
                if default is not None:
                    self._set(section, option, default)
                return default
            return self._config.get(section, option)

    def get_datetime(self, section, option, default=None):
        result = self.get(section, option, default)
        if result:
            return WSDateTime.from_csv(result)
        return result

    def set(self, section, option, value):
        """Set option in section to string value."""
        with self._lock:
            self._set(section, option, value)

    def _set(self, section, option, value):
        if not self._config.has_section(section):
            self._config.add_section(section)
        elif (self._config.has_option(section, option) and
              self._config.get(section, option) == value):
            return
        self._config.set(section, option, value)
        self._dirty = True

    def unset(self, section, option):
        """Remove option from section."""
        with self._lock:
            if not self._config.has_section(section):
                return
            if self._config.has_option(section, option):
                self._config.remove_option(section, option)
                self._dirty = True
            if not self._config.options(section):
                self._config.remove_section(section)
                self._dirty = True

class PywwsContext(object):
    def __init__(self, data_dir, live_logging):
        self.live_logging = live_logging
        # open params and status files
        self.params = ParamStore(data_dir, 'weather.ini')
        self.status = ParamStore(data_dir, 'status.ini')
        # update weather.ini
        self.update_params()
        # create working directories
        self.work_dir = self.params.get('paths', 'work', '/tmp/pywws')
        self.output_dir = os.path.join(self.work_dir, 'output')
        if not os.path.isdir(self.output_dir):
            os.makedirs(self.output_dir)
        # Load whichever data store module was specified,
        # Defaults to the original file store
        datastoretype = self.params.get('paths', 'datastoretype', 'filedata')
        DataStoreModule = importlib.import_module(
            '.'+datastoretype, package='pywws')
        # open data file stores
        self.raw_data = DataStoreModule.RawStore(data_dir)
        self.calib_data = DataStoreModule.CalibStore(data_dir)
        self.hourly_data = DataStoreModule.HourlyStore(data_dir)
        self.daily_data = DataStoreModule.DailyStore(data_dir)
        self.monthly_data = DataStoreModule.MonthlyStore(data_dir)
        # create an event to shutdown threads
        self.shutdown = threading.Event()

    def update_params(self):
        # convert day end hour
        day_end_str = self.params.get('config', 'day end hour')
        if day_end_str and not ',' in day_end_str:
            logger.error('updating "day end hour" in weather.ini')
            day_end_str += ', False'
            self.params.set('config', 'day end hour', day_end_str)
        # convert uploads to use pywws.service.{copy,ftp,sftp,twitter}
        local_site = self.params.get('ftp', 'local site')
        secure = self.params.get('ftp', 'secure')
        privkey = self.params.get('ftp', 'privkey')
        if local_site or secure or privkey:
            if local_site == 'True':
                self.params.set(
                    'copy', 'directory', self.params.get('ftp', 'directory', ''))
                mod = 'copy'
            elif secure == 'True':
                for key in ('site', 'user', 'directory', 'port',
                            'password', 'privkey'):
                    self.params.set('sftp', key, self.params.get('ftp', key, ''))
                mod = 'sftp'
            else:
                mod = 'ftp'
            for key in ('local site', 'secure', 'privkey'):
                self.params.unset('ftp', key)
            for section in self.params._config.sections():
                if section.split()[0] != 'cron' and section not in [
                        'live', 'logged', 'hourly', '12 hourly', 'daily']:
                    continue
                for t_p in ('text', 'plot'):
                    templates = literal_eval(
                        self.params.get(section, t_p, '[]'))
                    services = literal_eval(
                        self.params.get(section, 'services', '[]'))
                    changed = False
                    for n, template in enumerate(templates):
                        if isinstance(template, (list, tuple)):
                            template, flags = template
                            templates[n] = template
                            changed = True
                        else:
                            flags = ''
                        if t_p == 'plot':
                            result = os.path.splitext(template)[0]
                        else:
                            result = template
                        if 'L' in flags:
                            task = None
                        elif 'T' in flags:
                            task = ('twitter', result)
                        else:
                            task = (mod, result)
                        if task and task not in services:
                            services.append(task)
                            changed = True
                    if changed:
                        logger.error('updating %s in [%s]', t_p, section)
                        self.params.set(section, t_p, repr(templates))
                        self.params.set(section, 'services', repr(services))
        self.params.unset('config', 'config version')

    def terminate(self):
        if self.live_logging:
            # signal threads to terminate
            self.shutdown.set()
        # wait for threads to terminate
        for thread in threading.enumerate():
            if thread == threading.current_thread():
                continue
            logger.debug('waiting for thread ' + thread.name)
            thread.join()

    def flush(self):
        logger.debug('flushing')
        self.params.flush()
        self.status.flush()
        self.raw_data.flush()
        self.calib_data.flush()
        self.hourly_data.flush()
        self.daily_data.flush()
        self.monthly_data.flush()


@contextmanager
def pywws_context(data_dir, live_logging=False):
    ctx = PywwsContext(data_dir, live_logging)
    try:
        yield ctx
    finally:
        ctx.terminate()
        ctx.flush()



#!/usr/bin/env python

# pywws - Python software for USB Wireless Weather Stations
# http://github.com/jim-easterbrook/pywws
# Copyright (C) 2008-18  pywws contributors

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Provide a couple of :py:class:`datetime.tzinfo` compatible objects
representing local time and UTC.

Introduction
------------

This module provides two :py:class:`datetime.tzinfo` compatible objects
representing UTC and local time zones. These are used to convert
timestamps to and from UTC and local time. The weather station software
stores data with UTC timestamps, to avoid problems with daylight savings
time, but the template and plot programs output data with local times.

Detailed API
------------

"""

from __future__ import print_function

from datetime import datetime, timedelta
import logging
import sys

import pytz
import tzlocal

from pywws.constants import DAY, HOUR

logger = logging.getLogger(__name__)


class TimeZone(object):
    def __init__(self, tz_name=None):
        if tz_name:
            self.local = pytz.timezone(tz_name)
        else:
            self.local = tzlocal.get_localzone()
        logger.info('Using timezone "{!s}"'.format(self.local))
        self.utc = pytz.utc
        now = datetime.utcnow().replace(day=15, hour=12)
        while self.dst(now):
            now -= timedelta(days=30)
        self.standard_offset = self.utcoffset(now)

    def dst(self, dt):
        return self.local.dst(dt, is_dst=False)

    def utcoffset(self, dt):
        return self.local.utcoffset(dt, is_dst=False)

    def localize(self, dt):
        """Attach local timezone to a naive timestamp with no adjustment."""
        return self.local.localize(dt)

    def to_local(self, dt):
        """Convert any timestamp to local time (with tzinfo)."""
        if dt.tzinfo is None:
            dt = dt.replace(tzinfo=self.utc)
        return dt.astimezone(self.local)

    def to_utc(self, dt):
        """Convert any timestamp to UTC (with tzinfo)."""
        if dt.tzinfo is None:
            return dt.replace(tzinfo=self.utc)
        return dt.astimezone(self.utc)

    def to_naive(self, dt):
        """Convert any timestamp to pywws (utc, no tzinfo)."""
        if dt.tzinfo is None:
            return dt
        return dt.astimezone(self.utc).replace(tzinfo=None)

    def local_replace(self, dt, use_dst=True, _recurse=False, **kwds):
        """Return pywws timestamp (utc, no tzinfo) for the most recent
        local time before the pywws timestamp dt, with datetime replace
        applied.

        """
        local_time = dt + self.standard_offset
        if use_dst:
            dst_offset = self.dst(local_time)
            if dst_offset:
                local_time += dst_offset
                adjusted_time = local_time.replace(**kwds)
                if adjusted_time > local_time and not _recurse:
                    return self.local_replace(
                        dt - DAY, use_dst=use_dst, _recurse=True, **kwds)
                adjusted_time -= dst_offset
                if self.dst(adjusted_time):
                    return adjusted_time - self.standard_offset
        adjusted_time = local_time.replace(**kwds)
        if use_dst:
            dst_offset = self.dst(adjusted_time)
            adjusted_time -= dst_offset
        if adjusted_time > local_time and not _recurse:
            return self.local_replace(
                dt - DAY, use_dst=use_dst, _recurse=True, **kwds)
        return adjusted_time - self.standard_offset

    def local_midnight(self, dt):
        """Return pywws timestamp (utc, no tzinfo) for the local time
        midnight before the supplied pywws timestamp.

        """
        return self.local_replace(dt, hour=0, minute=0, second= 0)


timezone = TimeZone()


def main():
    now = datetime.utcnow().replace(microsecond=0)
    print('using timezone', timezone.local)
    print('current time')
    print(now, 'UTC')
    lcl = timezone.to_local(now)
    print(lcl, lcl.strftime('%Z'))
    now_utc = timezone.to_utc(lcl)
    print(now_utc, now_utc.strftime('%Z'))
    print()
    print('most recent 9am',
          timezone.local_replace(now, hour=9, minute=0, second=0), 'UTC')
    print('most recent 9pm',
          timezone.local_replace(now, hour=21, minute=0, second=0), 'UTC')
    print()
    print('DST transitions in "America/St_Johns"')
    tz = TimeZone('America/St_Johns')
    for day in range(11, 13):
        dt = datetime(2018, 3, day, 12)
        mn = tz.local_midnight(dt)
        lcl = tz.to_local(mn)
        print('midnight', mn, 'UTC', lcl, lcl.strftime('%Z'))
    print()
    for day in range(4, 6):
        dt = datetime(2018, 11, day, 12)
        mn = tz.local_midnight(dt)
        lcl = tz.to_local(mn)
        print('midnight', mn, 'UTC', lcl, lcl.strftime('%Z'))


if __name__ == "__main__":
    sys.exit(main())



# pywws - Python software for USB Wireless Weather Stations
# http://github.com/jim-easterbrook/pywws
# Copyright (C) 2018  pywws contributors

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Upload weather data to wetter.com.

wetter.com_ is a web site gathering data mainly from stations in
Germany. It's referred to in pywws as wetterarchivde for historical
reasons.

* Register station: https://www.wetter.com/mein_wetter/login/
* Additional dependency: http://docs.python-requests.org/
* Example ``weather.ini`` configuration::

    [wetterarchivde]
    user_id = 12345
    kennwort = ab1d3456i8

    [logged]
    services = ['wetterarchivde', 'underground']

    [live]
    services = ['wetterarchivde', 'underground']

.. _wetter.com: http://netzwerk.wetter.com/

"""

from __future__ import absolute_import, unicode_literals

from contextlib import contextmanager
from datetime import timedelta
import logging
import os
import sys

import requests

import pywws.service

__docformat__ = "restructuredtext en"
service_name = os.path.splitext(os.path.basename(__file__))[0]
logger = logging.getLogger(__name__)


class ToService(pywws.service.CatchupDataService):
    config = {
        'user_id' : ('', True, 'id'),
        'kennwort': ('', True, 'pwd'),
        }
    fixed_data = {'sid': 'pywws'}
    interval = timedelta(seconds=300)
    logger = logger
    service_name = service_name
    template = """
#live#
#roundtime True#
#idx                    "'dtutc': '%Y%m%d%H%M',"#
#timezone local#
#idx                    "'dt': '%Y%m%d%H%M',"#
#hum_out                "'hu': '%.d',"#
#temp_out               "'te': '%.1f',"#
#rel_pressure           "'pr': '%.1f',"#
#wind_dir               "'wd': '%.0f'," "" "winddir_degrees(x)"#
#wind_ave               "'ws': '%.1f',"#
#wind_gust              "'wg': '%.1f',"#
#calc "rain_hour(data)" "'pa': '%.1f',"#
"""

    def __init__(self, context, check_params=True):
        super(ToService, self).__init__(context, check_params)
        # extend template
        if context.params.get('config', 'ws type') == '3080':
            self.template += """#uv "'uv': '%d',"#"""

    @contextmanager
    def session(self):
        with requests.Session() as session:
            yield session

    def upload_data(self, session, prepared_data={}):
        try:
            rsp = session.post('http://interface.wetterarchiv.de/weather/',
                               data=prepared_data, timeout=60)
        except Exception as ex:
            return False, repr(ex)
        if rsp.status_code != 200:
            return False, 'http status: {:d}'.format(rsp.status_code)
        rsp = rsp.json()
        if rsp:
            return True, 'server response "{!r}"'.format(rsp)
        return True, 'OK'


if __name__ == "__main__":
    sys.exit(pywws.service.main(ToService))



# pywws - Python software for USB Wireless Weather Stations
# http://github.com/jim-easterbrook/pywws
# Copyright (C) 2008-18  pywws contributors

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Upload files to a web server by SFTP.

This module uploads files to (typically) a website *via* SFTP. Details
of the upload destination are stored in the file ``weather.ini`` in your
data directory. You should be able to get the required information from
your web space provider. If your provider doesn't allow SFTP then use
:py:mod:`pywws.service.ftp` instead.

* Additional dependency: https://www.paramiko.org/
* Example ``weather.ini`` configuration::

    [sftp]
    site = ftp.xxxx.yyyy.co.uk
    user = xxxxxxx
    directory = public_html/weather/data/
    port = 22
    password =
    privkey = /home/pywws/.ssh/webhost_rsa

    [hourly]
    plot = ['24hrs.png.xml', 'rose_12hrs.png.xml']
    text = ['24hrs.txt']
    services = [('sftp', '24hrs.txt', '24hrs.png', 'rose_12hrs.png')]

Paramiko can be installed with ``pip``::

    sudo pip install paramiko

Run :py:mod:`pywws.service.sftp` once to set the default configuration,
which you can then change. ``directory`` is the name of a directory in
which all the uploaded files will be put. This will depend on the
structure of your web site and the sort of host you use. ``port`` is the
port number to use. 22 is the standard value but your web space provider
may require a different port.

Authentication can be by password or RSA public key. To use a key you
first need to create a passwordless key pair using ``ssh-keygen``, then
copy the public key to your web space provider. For example::

    ssh-keygen -t rsa -f webhost_rsa
    ssh-copy-id -i webhost_rsa.pub xxxxxxx@ftp.xxxx.yyyy.co.uk

Move both key files to somewhere convenient, such as
``/home/pywws/.ssh/`` and set ``privkey`` to the full path of the
private key.

You can upload any files you like, as often as you like, but typical
usage is to update a website once an hour. Each file to be uploaded
needs to be listed in a service entry like ``('sftp', 'filename')``. If
the file is not in your ``work`` directory's ``output`` directory then
``filename`` should be the full path.

"""

from __future__ import absolute_import

from contextlib import contextmanager
from datetime import timedelta
import logging
import os
import sys

import paramiko

import pywws.service

__docformat__ = "restructuredtext en"
service_name = os.path.splitext(os.path.basename(__file__))[0]
logger = logging.getLogger(__name__)


class ToService(pywws.service.FileService):
    config = {
        'site'       : ('',   True,  None),
        'user'       : ('',   True,  None),
        'password'   : ('',   False, None),
        'directory'  : ('',   True,  None),
        'port'       : ('22', True,  None),
        'privkey'    : ('',   False, None),
        }
    logger = logger
    service_name = service_name

    def __init__(self, context, check_params=True):
        super(ToService, self).__init__(context, check_params)
        self.params['port'] = int(self.params['port'])
        if self.params['privkey']:
            self.params['privkey'] = paramiko.RSAKey.from_private_key_file(
                self.params['privkey'])

    @contextmanager
    def session(self):
        logger.info("Uploading to web site with SFTP")
        address = (self.params['site'], self.params['port'])
        with paramiko.Transport(address) as transport:
            transport.start_client(timeout=30)
            if self.params['privkey']:
                transport.auth_publickey(username=self.params['user'],
                                         key=self.params['privkey'])
            else:
                transport.auth_password(username=self.params['user'],
                                        password=self.params['password'])
            with paramiko.SFTPClient.from_transport(transport) as session:
                session.get_channel().settimeout(20)
                session.chdir(self.params['directory'])
                yield session

    def upload_file(self, session, path):
        target = os.path.basename(path)
        try:
            session.put(path, target)
        except Exception as ex:
            return False, repr(ex)
        return True, 'OK'


if __name__ == "__main__":
    sys.exit(pywws.service.main(ToService))



# pywws - Python software for USB Wireless Weather Stations
# http://github.com/jim-easterbrook/pywws
# Copyright (C) 2008-18  pywws contributors

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Test quality of USB connection to weather station

::

%s

The USB link to my weather station is not 100%% reliable. The data
read from the station by the computer is occasionally corrupted,
perhaps by interference. I've been trying to solve this by putting
ferrite beads around the USB cable and relocating possible
interference sources such as external hard drives. All without any
success so far.

This program tests the USB connection for errors by continuously
reading the entire weather station memory (except for those parts that
may be changing) looking for errors. Each 32-byte block is read twice,
and if the two readings differ a warning message is displayed. Also
displayed are the number of blocks read, and the number of errors
found.

I typically get one or two errors per hour, so the test needs to be
run for several hours to produce a useful measurement. Note that other
software that accesses the weather station (such as
:py:mod:`pywws.hourly` or :py:mod:`pywws.livelog`)
must not be run while the test is in progress.

If you run this test and get no errors at all, please let me know.
There is something good about your setup and I'd love to know what it
is!

"""

from __future__ import absolute_import, print_function

__docformat__ = "restructuredtext en"

__usage__ = """
 usage: python -m pywws.usbtest [options]
 options are:
  -h | --help           display this help
  -v | --verbose        increase amount of reassuring messages
                        (repeat for even more messages e.g. -vvv)
"""

__doc__ %= __usage__
__usage__ = __doc__.split('\n')[0] + __usage__

import getopt
import sys

import pywws.logger
import pywws.weatherstation


def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        opts, args = getopt.getopt(argv[1:], "hv", ('help', 'verbose'))
    except getopt.error as msg:
        print('Error: %s\n' % msg, file=sys.stderr)
        print(__usage__.strip(), file=sys.stderr)
        return 1
    # check arguments
    if len(args) != 0:
        print('Error: no arguments allowed\n', file=sys.stderr)
        print(__usage__.strip(), file=sys.stderr)
        return 2
    # process options
    verbose = 0
    for o, a in opts:
        if o in ('-h', '--help'):
            print(__usage__.strip())
            return 0
        elif o in ('-v', '--verbose'):
            verbose += 1
    # do it!
    pywws.logger.setup_handler(verbose)
    ws = pywws.weatherstation.WeatherStation()
    fixed_block = ws.get_fixed_block()
    if not fixed_block:
        print("No valid data block found")
        return 3
    # loop
    ptr = ws.data_start
    total_count = 0
    bad_count = 0
    while True:
        if total_count % 1000 == 0:
            active = ws.current_pos()
        while True:
            ptr += 0x20
            if ptr >= 0x10000:
                ptr = ws.data_start
            if active < ptr - 0x10 or active >= ptr + 0x20:
                break
        result_1 = ws._read_block(ptr, retry=False)
        result_2 = ws._read_block(ptr, retry=False)
        if result_1 != result_2:
            logger.warning('read_block changing %06x', ptr)
            logger.warning('  %s', str(result_1))
            logger.warning('  %s', str(result_2))
            bad_count += 1
        total_count += 1
        print("\r %d/%d " % (bad_count, total_count), end='', flush=True)
    print('')
    return 0


if __name__ == "__main__":
    try:
        sys.exit(main())
    except KeyboardInterrupt:
        pass



# pywws - Python software for USB Wireless Weather Stations
# http://github.com/jim-easterbrook/pywws
# Copyright (C) 2018  pywws contributors

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Upload weather data to an AWS Gateway API (or other) service.

This is for any service that accepts key value data via a GET method. This
supports optional extra headers.

* Additional dependency: http://docs.python-requests.org/
* Example ``weather.ini`` configuration::

    [aws_api_gw_service]
    api url	= https://my-aws-api-gw.execute-api.eu-west-1.amazonaws.com/weather
    http headers	= [('x-api-key', 'my-api-key'), ('another-one', 'spam-val')]

    [logged]
    services = ['aws_api_gw_service', 'metoffice']

    [live]
    services = ['aws_api_gw_service', 'metoffice']

"""

from __future__ import absolute_import, unicode_literals

from ast import literal_eval
from contextlib import contextmanager
from datetime import timedelta
import logging
import os
import sys

import requests

import pywws
from pywws.conversions import rain_inch
import pywws.service

__docformat__ = "restructuredtext en"
service_name = os.path.splitext(os.path.basename(__file__))[0]
logger = logging.getLogger(__name__)


class ToService(pywws.service.CatchupDataService):
    catchup = 100
    config = {
        'http headers': (None, False, None),
        'api url'     : ('',   True,  None),
        }
    interval = timedelta(seconds=300)
    logger = logger
    service_name = service_name
    template = """
#live#
#hum_in       "'hum_in'     : '%.d',"#
#temp_in      "'temp_in'    : '%.1f',"#
#hum_out      "'hum_out'    : '%.d',"#
#temp_out     "'temp_out'   : '%.1f',"#
#calc "dew_point(data['temp_out'], data['hum_out'])" "'temp_dewpt'  : '%.1f',"#
#rel_pressure "'abs_pressure'   : '%.4f',"#
#wind_ave     "'wind_ave'   : '%.2f'," "" "wind_mph(x)"#
#wind_gust    "'wind_gust'  : '%.2f'," "" "wind_mph(x)"#
#wind_dir     "'wind_dir'   : '%.0f'," "" "winddir_degrees(x)"#
#calc "rain_hour(data)" "'rain'     : '%g',"#
#calc "rain_day(data)" "'rain_day'     : '%g',"#
#calc "wind_chill(data['temp_out'], data['wind_ave'])" "'wind_chill'	: '%.1f',"#
#calc "apparent_temp(data['temp_out'], data['hum_out'], data['wind_ave'])" "'temp_apprt'	:  '%.1f',"#
#calc "cloud_base(data['temp_out'], data['hum_out'])" "'cloud_base' : '%.1f',"#
#idx          "'tdate'      : '%Y-%m-%d',"#
#idx          "'ttime'      : '%H:%M:%S',"#

"""

    def __init__(self, context, check_params=True):
        super(ToService, self).__init__(context, check_params)
        # initialise rain history
        last_update = context.status.get_datetime('last update', service_name)
        if last_update:
            last_update = context.calib_data.nearest(last_update)
            self.last_rain = context.calib_data[last_update]['rain']
        else:
            self.last_rain = None

    @contextmanager
    def session(self):
        with requests.Session() as session:
            yield session

    def upload_data(self, session, prepared_data={}):
        try:
            if self.params['http headers']:
                for header in literal_eval(self.params['http headers']):
                    session.headers.update({header[0]: header[1]})
            rsp = session.get(self.params['api url'], params=prepared_data,
                timeout=60)
        except Exception as ex:
            return False, repr(ex)
        if rsp.status_code != 200:
            return False, 'http status: {:d}'.format(rsp.status_code)
        rsp = rsp.json()
        if rsp:
            return True, 'server response "{!r}"'.format(rsp)
        return True, 'OK'


if __name__ == "__main__":
    sys.exit(pywws.service.main(ToService))



# pywws - Python software for USB Wireless Weather Stations
# http://github.com/jim-easterbrook/pywws
# Copyright (C) 2018  pywws contributors

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Upload current temperature to temperatur.nu.

temperatur.nu is a Swedish web site that shows the current temperature
in many places.

* Web site: http://www.temperatur.nu/
* Additional dependency: http://docs.python-requests.org/
* Example ``weather.ini`` configuration::

    [temperaturnu]
    hash = longhexnumber

    [live]
    services = ['temperaturnu', 'underground']

    [logged]
    services = ['temperaturnu', 'underground']

You receive the hash value from the temperatur.nu admins during sign
up.  It looks like ``d3b07384d113edec49eaa6238ad5ff00``.

.. _temperatur.nu: http://www.temperatur.nu/

"""

from __future__ import absolute_import, unicode_literals

from contextlib import contextmanager
from datetime import timedelta
import logging
import os
import sys

import requests

import pywws.service

__docformat__ = "restructuredtext en"
service_name = os.path.splitext(os.path.basename(__file__))[0]
logger = logging.getLogger(__name__)


class ToService(pywws.service.LiveDataService):
    config = {'hash': ('', True, 'hash')}
    logger = logger
    service_name = service_name
    template = "#live##temp_out \"'t': '%.1f',\"#"

    @contextmanager
    def session(self):
        with requests.Session() as session:
            yield session

    def valid_data(self, data):
        return data['temp_out'] is not None

    def upload_data(self, session, prepared_data={}):
        try:
            rsp = session.get('http://www.temperatur.nu/rapportera.php',
                              params=prepared_data, timeout=60)
        except Exception as ex:
            return False, repr(ex)
        if rsp.status_code != 200:
            return False, 'http status: {:d}'.format(rsp.status_code)
        text = rsp.text.strip()
        if text:
            return True, 'server response "{:s}"'.format(text)
        return True, 'OK'


if __name__ == "__main__":
    sys.exit(pywws.service.main(ToService))



#!/usr/bin/env python

# pywws - Python software for USB Wireless Weather Stations
# http://github.com/jim-easterbrook/pywws
# Copyright (C) 2008-18  pywws contributors

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Run 'live logging' as a UNIX daemon.

This script can also be run with the ``pywws-livelog-daemon`` command. ::
%s
Requires the python-daemon library.

If you get a "function() argument 1 must be code, not str" error, try
installing python-daemon from PyPI instead of your Linux repos.

For more information on 'live logging' see :doc:`../guides/livelogging`.

"""

from __future__ import absolute_import, print_function

__docformat__ = "restructuredtext en"
__usage__ = """
 usage: %s [options] data_dir log_file start|stop|restart
 options are:
  -h      or --help        display this help
  -p file or --pid file    store pid in 'file' (default /run/lock/pywws.pid)
  -v      or --verbose     increase amount of logging messages
 data_dir is the root directory of the weather data (e.g. ~/weather/data)
 log_file is a file to write logging to, e.g. /var/log/pywws.log
"""
__doc__ %= __usage__ % ('python -m pywws.livelogdaemon')

import getopt
import logging
import os
import sys

from daemon.daemon import DaemonContext
from daemon.runner import DaemonRunner, make_pidlockfile

import pywws.livelog
import pywws.logger

logger = logging.getLogger(__name__)

class PatchedDaemonRunner(DaemonRunner):
    # modify DaemonRunner to work with Python3
    def __init__(self, app):
        self.parse_args()
        self.app = app
        self.daemon_context = DaemonContext()
        self.daemon_context.stdin = open(app.stdin_path, 'rt')
        self.daemon_context.stdout = open(app.stdout_path, 'w+t')
        if sys.version_info[0] >= 3:
            self.daemon_context.stderr = open(
                    app.stderr_path, 'w+b', buffering=0)
        else:
            self.daemon_context.stderr = open(
                    app.stderr_path, 'w+t', buffering=0)

        self.pidfile = None
        if app.pidfile_path is not None:
            self.pidfile = make_pidlockfile(
                    app.pidfile_path, app.pidfile_timeout)
        self.daemon_context.pidfile = self.pidfile


class App(object):
    # attributes required by daemon runner
    stdin_path = '/dev/null'
    stdout_path = '/dev/null'
    stderr_path = '/dev/null'
    pidfile_path = '/run/lock/pywws.pid'
    pidfile_timeout = 5

    def __init__(self, argv):
        usage = (__usage__ % (argv[0])).strip()
        try:
            opts, args = getopt.getopt(
                argv[1:], "hp:v", ['help', 'pid=', 'verbose'])
        except getopt.error as msg:
            print('Error: %s\n' % msg, file=sys.stderr)
            print(usage, file=sys.stderr)
            sys.exit(1)
        # process options
        self.verbose = 0
        for o, a in opts:
            if o in ('-h', '--help'):
                print(__doc__.split('\n\n')[0])
                print(usage)
                sys.exit(0)
            elif o in ('-p', '--pid'):
                self.pidfile_path = os.path.abspath(a)
            elif o in ('-v', '--verbose'):
                self.verbose += 1
        # check arguments
        if len(args) != 3:
            print('Error: 3 arguments required\n', file=sys.stderr)
            print(usage, file=sys.stderr)
            sys.exit(2)
        self.data_dir = os.path.abspath(args[0])
        self.logfile = os.path.abspath(args[1])
        # leave remaining argument for daemon runner to parse
        sys.argv = [argv[0], args[2]]

    def run(self):
        pywws.logger.setup_handler(self.verbose, self.logfile)
        try:
            pywws.livelog.live_log(self.data_dir)
        except Exception as ex:
            logger.exception(ex)


def main(argv=None):
    if argv is None:
        argv = sys.argv
    runner = PatchedDaemonRunner(App(argv))
    runner.do_action()
    return 0

if __name__ == "__main__":
    sys.exit(main())
